<?php

namespace Drupal\relative_to_absolute_urls\Normalizer;

use Drupal\Component\Render\MarkupInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Render\Markup;
use Drupal\views\Render\ViewsRenderPipelineMarkup;
use Drupal\serialization\Normalizer\MarkupNormalizer;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Normalizer for Markup and ViewsRenderPipelineMarkup fields.
 */
class AbsoluteDataNormalizer extends MarkupNormalizer {

  /**
   * The supported interface or class.
   *
   * @var string[]
   */
  protected $supportedInterfaceOrClass = [
    ViewsRenderPipelineMarkup::class,
    Markup::class,
    MarkupInterface::class,
  ];

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs a new AbsoluteDataNormalizer.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(RequestStack $request_stack) {
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public function supportsNormalization($data, ?string $format = NULL, array $context = []): bool {
    return ($data instanceof ViewsRenderPipelineMarkup || $data instanceof Markup)
      && $format === 'json';
  }

  /**
   * {@inheritdoc}
   */
  public function normalize($object, $format = NULL, array $context = []): string {
    // First let parent handle basic normalization.
    $normalized = parent::normalize($object, $format, $context);

    // Get base URL from current request.
    $base_url = $this->requestStack->getCurrentRequest()->getSchemeAndHttpHost();

    // Convert relative URLs to absolute.
    return Html::transformRootRelativeUrlsToAbsolute($normalized, $base_url);
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedTypes(?string $format): array {
    return [
      ViewsRenderPipelineMarkup::class => TRUE,
      Markup::class => TRUE,
      MarkupInterface::class => TRUE,
    ];
  }

}
